@@ -42,7 +42,10 @@
 #ifdef CVTUTF_DEBUG
  #include <stdio.h>
 #endif
-#include <string.h> /* strlen() */
+#include <string.h> /* strlen () */
+
+#include "transmission.h"
+
 #include "ConvertUTF.h"
 
 static const int halfShift  = 10; /* used for shifting by 10 bits */
@@ -50,10 +53,10 @@ static const int halfShift  = 10; /* used for shifting by 10 bits */
 static const UTF32 halfBase = 0x0010000UL;
 static const UTF32 halfMask = 0x3FFUL;
 
-#define UNI_SUR_HIGH_START  (UTF32)0xD800
-#define UNI_SUR_HIGH_END    (UTF32)0xDBFF
-#define UNI_SUR_LOW_START   (UTF32)0xDC00
-#define UNI_SUR_LOW_END     (UTF32)0xDFFF
+#define UNI_SUR_HIGH_START (UTF32)0xD800
+#define UNI_SUR_HIGH_END  (UTF32)0xDBFF
+#define UNI_SUR_LOW_START (UTF32)0xDC00
+#define UNI_SUR_LOW_END   (UTF32)0xDFFF
 #define false	   0
 #define true	    1
 
@@ -156,8 +159,8 @@ ConversionResult ConvertUTF16toUTF32 (
     *targetStart = target;
 #ifdef CVTUTF_DEBUG
 if (result == sourceIllegal) {
-    fprintf(stderr, "ConvertUTF16toUTF32 illegal seq 0x%04x,%04x\n", ch, ch2);
-    fflush(stderr);
+    fprintf (stderr, "ConvertUTF16toUTF32 illegal seq 0x%04x,%04x\n", ch, ch2);
+    fflush (stderr);
 }
 #endif
     return result;
@@ -193,7 +196,7 @@ static const UTF32 offsetsFromUTF8[6] = { 0x00000000UL, 0x00003080UL, 0x000E2080
 
 /*
  * Once the bits are split out into bytes of UTF-8, this is a mask OR-ed
- * into the first byte, depending on how many bytes follow.  There are
+ * into the first byte, depending on how many bytes follow. There are
  * as many entries in this table as there are UTF-8 sequence types.
  * (I.e., one byte sequence, two byte... etc.). Remember that sequencs
  * for *legal* UTF-8 will be 4 or fewer bytes total.
@@ -271,7 +274,7 @@ ConversionResult ConvertUTF16toUTF8 (
 	    case 4: *--target = (UTF8)((ch | byteMark) & byteMask); ch >>= 6;
 	    case 3: *--target = (UTF8)((ch | byteMark) & byteMask); ch >>= 6;
 	    case 2: *--target = (UTF8)((ch | byteMark) & byteMask); ch >>= 6;
-	    case 1: *--target =  (UTF8)(ch | firstByteMark[bytesToWrite]);
+	    case 1: *--target = (UTF8)(ch | firstByteMark[bytesToWrite]);
 	}
 	target += bytesToWrite;
     }
@@ -289,11 +292,11 @@ ConversionResult ConvertUTF16toUTF8 (
  *  length = trailingBytesForUTF8[*source]+1;
  * and the sequence is illegal right away if there aren't that many bytes
  * available.
- * If presented with a length > 4, this returns false.  The Unicode
+ * If presented with a length > 4, this returns false. The Unicode
  * definition of UTF-8 goes up to 4-byte sequences.
  */
 
-static Boolean isLegalUTF8(const UTF8 *source, int length) {
+static Boolean isLegalUTF8 (const UTF8 *source, int length) {
     UTF8 a;
     const UTF8 *srcptr = source+length;
     switch (length) {
@@ -301,12 +304,12 @@ static Boolean isLegalUTF8(const UTF8 *source, int length) {
 	/* Everything else falls through when "true"... */
     case 4: if ((a = (*--srcptr)) < 0x80 || a > 0xBF) return false;
     case 3: if ((a = (*--srcptr)) < 0x80 || a > 0xBF) return false;
-    case 2: if ((a = (*--srcptr)) > 0xBF) return false;
+    case 2: if ((a = (*--srcptr)) < 0x80 || a > 0xBF) return false;
 
 	switch (*source) {
 	    /* no fall-through in this inner switch */
 	    case 0xE0: if (a < 0xA0) return false; break;
-	    case 0xED: if ((a < 0x80) || (a > 0x9F)) return false; break;
+	    case 0xED: if (a > 0x9F) return false; break;
 	    case 0xF0: if (a < 0x90) return false; break;
 	    case 0xF4: if (a > 0x8F) return false; break;
 	    default:   if (a < 0x80) return false;
@@ -325,7 +328,7 @@ static Boolean isLegalUTF8(const UTF8 *source, int length) {
  * This is not used here; it's just exported.
  */
 
-Boolean isLegalUTF8Sequence(const UTF8 *source, const UTF8 *sourceEnd) {
+Boolean isLegalUTF8Sequence (const UTF8 *source, const UTF8 *sourceEnd) {
     int length;
     if (source == sourceEnd) {
         return true;
@@ -335,7 +338,7 @@ Boolean isLegalUTF8Sequence(const UTF8 *source, const UTF8 *sourceEnd) {
         if (source+length > sourceEnd) {
             return false;
         }
-        if (!isLegalUTF8(source, length)) {
+        if (!isLegalUTF8 (source, length)) {
             return false;
         }
         source += length;
@@ -346,52 +349,50 @@ Boolean isLegalUTF8Sequence(const UTF8 *source, const UTF8 *sourceEnd) {
 }
 
 /**
- * This is a variation of isLegalUTF8Sequence() that behaves like g_utf8_validate().
+ * This is a variation of isLegalUTF8Sequence () that behaves like g_utf8_validate ().
  * In addition to knowing if the sequence is legal, it also tells you the last good character.
  */
 Boolean
-tr_utf8_validate( const char * str, int max_len, const char ** end )
+tr_utf8_validate (const char * str, size_t max_len, const char ** end)
 {
     const UTF8* source = (const UTF8*) str;
-    const UTF8* sourceEnd = source;
+    const UTF8* sourceEnd;
 
-    if( max_len == 0 )
+    if (max_len == 0)
         return true;
 
-    if( str == NULL )
+    if (str == NULL)
         return false;
 
-    sourceEnd = source + ((max_len < 0) ? strlen(str) : (size_t)max_len);
+    sourceEnd = source + (max_len == TR_BAD_SIZE ? strlen (str) : max_len);
 
-    if( source == sourceEnd )
+    if (source == sourceEnd)
     {
-        if( end != NULL )
+        if (end != NULL)
             *end = (const char*) source;
         return true;
     }
 
-    for( ;; )
+    for (;;)
     {
         const int length = trailingBytesForUTF8[*source] + 1;
         if (source + length > sourceEnd) {
-            if( end != NULL )
+            if (end != NULL)
                 *end = (const char*) source;
             return false;
         }
-        if (!isLegalUTF8(source, length)) {
-            if( end != NULL )
+        if (!isLegalUTF8 (source, length)) {
+            if (end != NULL)
                 *end = (const char*) source;
             return false;
         }
         source += length;
         if (source >= sourceEnd) {
-            if( end != NULL )
+            if (end != NULL)
                 *end = (const char*) source;
             return true;
         }
     }
-
-    
 }
 
 
@@ -406,11 +407,11 @@ ConversionResult ConvertUTF8toUTF16 (
     while (source < sourceEnd) {
 	UTF32 ch = 0;
 	unsigned short extraBytesToRead = trailingBytesForUTF8[*source];
-	if (source + extraBytesToRead >= sourceEnd) {
+	if (extraBytesToRead >= sourceEnd - source) {
 	    result = sourceExhausted; break;
 	}
 	/* Do this check whether lenient or strict */
-	if (! isLegalUTF8(source, extraBytesToRead+1)) {
+	if (! isLegalUTF8 (source, extraBytesToRead+1)) {
 	    result = sourceIllegal;
 	    break;
 	}
@@ -482,7 +483,7 @@ ConversionResult ConvertUTF32toUTF8 (
 	const UTF32 byteMask = 0xBF;
 	const UTF32 byteMark = 0x80; 
 	ch = *source++;
-	if (flags == strictConversion ) {
+	if (flags == strictConversion) {
 	    /* UTF-16 surrogate values are illegal in UTF-32 */
 	    if (ch >= UNI_SUR_HIGH_START && ch <= UNI_SUR_LOW_END) {
 		--source; /* return to the illegal value itself */
@@ -512,7 +513,7 @@ ConversionResult ConvertUTF32toUTF8 (
 	    case 4: *--target = (UTF8)((ch | byteMark) & byteMask); ch >>= 6;
 	    case 3: *--target = (UTF8)((ch | byteMark) & byteMask); ch >>= 6;
 	    case 2: *--target = (UTF8)((ch | byteMark) & byteMask); ch >>= 6;
-	    case 1: *--target = (UTF8) (ch | firstByteMark[bytesToWrite]);
+	    case 1: *--target = (UTF8)(ch | firstByteMark[bytesToWrite]);
 	}
 	target += bytesToWrite;
     }
@@ -532,11 +533,11 @@ ConversionResult ConvertUTF8toUTF32 (
     while (source < sourceEnd) {
 	UTF32 ch = 0;
 	unsigned short extraBytesToRead = trailingBytesForUTF8[*source];
-	if (source + extraBytesToRead >= sourceEnd) {
+	if (extraBytesToRead >= sourceEnd - source) {
 	    result = sourceExhausted; break;
 	}
 	/* Do this check whether lenient or strict */
-	if (! isLegalUTF8(source, extraBytesToRead+1)) {
+	if (! isLegalUTF8 (source, extraBytesToRead+1)) {
 	    result = sourceIllegal;
 	    break;
 	}
@@ -587,7 +588,7 @@ ConversionResult ConvertUTF8toUTF32 (
 
     Note A.
     The fall-through switches in UTF-8 reading code save a
-    temp variable, some decrements & conditionals.  The switches
+    temp variable, some decrements & conditionals. The switches
     are equivalent to the following loop:
 	{
 	    int tmpBytesToRead = extraBytesToRead+1;
