@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: BlocklistDownloader.h 7659 2009-01-10 23:37:37Z livings124 $
+ * $Id$
  *
- * Copyright (c) 2008-2009 Transmission authors and contributors
+ * Copyright (c) 2008-2012 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -33,12 +33,13 @@ typedef enum
     BLOCKLIST_DL_PROCESSING
 } blocklistDownloadState;
 
-@interface BlocklistDownloader : NSObject
+@interface BlocklistDownloader : NSObject <NSURLDownloadDelegate>
 {
     NSURLDownload * fDownload;
     
     BlocklistDownloaderViewController * fViewController;
     
+    NSString * fDestination;
     NSUInteger fCurrentSize;
     long long fExpectedSize;
     
