@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: DragOverlayView.h 7659 2009-01-10 23:37:37Z livings124 $
+ * $Id$
  *
- * Copyright (c) 2007-2009 Transmission authors and contributors
+ * Copyright (c) 2007-2012 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -26,7 +26,7 @@
 
 @interface DragOverlayView : NSView
 {
-    NSImage * fBadge, * fBackBadge;
+    NSImage * fBadge;
     
     NSDictionary * fMainLineAttributes, * fSubLineAttributes;
 }
