@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: ExpandedPathToIconTransformer.m 11617 2011-01-01 20:42:14Z livings124 $
+ * $Id$
  * 
- * Copyright (c) 2007-2011 Transmission authors and contributors
+ * Copyright (c) 2007-2012 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -45,7 +45,7 @@
     NSImage * icon;
     //show a folder icon if the folder doesn't exist
     if ([[path pathExtension] isEqualToString: @""] && ![[NSFileManager defaultManager] fileExistsAtPath: path])
-        icon = [[NSWorkspace sharedWorkspace] iconForFileType: NSFileTypeForHFSTypeCode('fldr')];
+        icon = [[NSWorkspace sharedWorkspace] iconForFileType: NSFileTypeForHFSTypeCode(kGenericFolderIcon)];
     else
         icon = [[NSWorkspace sharedWorkspace] iconForFile: path];
     
