@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: FileListNode.m 12483 2011-05-31 22:26:04Z livings124 $
+ * $Id$
  *
- * Copyright (c) 2008-2011 Transmission authors and contributors
+ * Copyright (c) 2008-2012 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -32,6 +32,16 @@
 
 @implementation FileListNode
 
+#warning remove ivars in header when 64-bit only (or it compiles in 32-bit mode)
+@synthesize name = fName;
+@synthesize path = fPath;
+@synthesize torrent = fTorrent;
+@synthesize size = fSize;
+@synthesize icon = fIcon;
+@synthesize isFolder = fIsFolder;
+@synthesize indexes = fIndexes;
+@synthesize children = fChildren;
+
 - (id) initWithFolderName: (NSString *) name path: (NSString *) path torrent: (Torrent *) torrent
 {
     if ((self = [self initWithFolder: YES name: name path: path torrent: torrent]))
@@ -80,51 +90,23 @@
     [fName release];
     [fPath release];
     [fIndexes release];
-    
     [fIcon release];
-    
     [fChildren release];
-    
     [super dealloc];
 }
 
 - (NSString *) description
 {
     if (!fIsFolder)
-        return [NSString stringWithFormat: @"%@ (%d)", fName, [fIndexes firstIndex]];
+        return [NSString stringWithFormat: @"%@ (%ld)", fName, [fIndexes firstIndex]];
     else
         return [NSString stringWithFormat: @"%@ (folder: %@)", fName, fIndexes];
 }
 
-- (BOOL) isFolder
-{
-    return fIsFolder;
-}
-
-- (NSString *) name
-{
-    return fName;
-}
-
-- (NSString *) path
-{
-    return fPath;
-}
-
-- (NSIndexSet *) indexes
-{
-    return fIndexes;
-}
-
-- (uint64_t) size
-{
-    return fSize;
-}
-
 - (NSImage *) icon
 {
     if (!fIcon)
-        fIcon = [[[NSWorkspace sharedWorkspace] iconForFileType: fIsFolder ? NSFileTypeForHFSTypeCode('fldr')
+        fIcon = [[[NSWorkspace sharedWorkspace] iconForFileType: fIsFolder ? NSFileTypeForHFSTypeCode(kGenericFolderIcon)
                                                                             : [fName pathExtension]] retain];
     return fIcon;
 }
@@ -136,9 +118,43 @@
     return fChildren;
 }
 
-- (Torrent *) torrent
+- (BOOL) updateFromOldName: (NSString *) oldName toNewName: (NSString *) newName inPath: (NSString *) path
 {
-    return fTorrent;
+    NSParameterAssert(oldName != nil);
+    NSParameterAssert(newName != nil);
+    NSParameterAssert(path != nil);
+    
+    NSArray * lookupPathComponents = [path pathComponents];
+    NSArray * thesePathComponents = [self.path pathComponents];
+    
+    if ([lookupPathComponents isEqualToArray: thesePathComponents]) //this node represents what's being renamed
+    {
+        if ([oldName isEqualToString: self.name])
+        {
+            [fName release];
+            fName = [newName copy];
+            return YES;
+        }
+    }
+    else if ([lookupPathComponents count] < [thesePathComponents count]) //what's being renamed is part of this node's path
+    {
+        lookupPathComponents = [lookupPathComponents arrayByAddingObject: oldName];
+        const BOOL allSame = NSNotFound == [lookupPathComponents indexOfObjectWithOptions: NSEnumerationConcurrent passingTest: ^BOOL(NSString * name, NSUInteger idx, BOOL * stop) {
+            return ![name isEqualToString: [thesePathComponents objectAtIndex: idx]];
+        }];
+        
+        if (allSame)
+        {
+            NSString * oldPathPrefix = [path stringByAppendingPathComponent: oldName];
+            NSString * newPathPrefix = [path stringByAppendingPathComponent: newName];
+            
+            [fPath autorelease];
+            fPath = [[fPath stringByReplacingCharactersInRange: NSMakeRange(0, [oldPathPrefix length]) withString: newPathPrefix] retain];
+            return YES;
+        }
+    }
+    
+    return NO;
 }
 
 @end
@@ -150,8 +166,8 @@
     if ((self = [super init]))
     {
         fIsFolder = isFolder;
-        fName = [name retain];
-        fPath = [path retain];
+        fName = [name copy];
+        fPath = [path copy];
         
         fIndexes = [[NSMutableIndexSet alloc] init];
         
