@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: FileOutlineController.h 7659 2009-01-10 23:37:37Z livings124 $
+ * $Id$
  *
- * Copyright (c) 2008-2009 Transmission authors and contributors
+ * Copyright (c) 2008-2012 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -30,7 +30,7 @@
 @interface FileOutlineController : NSObject
 {
     Torrent * fTorrent;
-    NSArray * fFileList;
+    NSMutableArray * fFileList;
     
     IBOutlet FileOutlineView * fOutline;
     
@@ -43,12 +43,16 @@
 
 - (void) setFilterText: (NSString *) text;
 
-- (void) reloadData;
+- (void) refresh;
 
 - (void) setCheck: (id) sender;
 - (void) setOnlySelectedCheck: (id) sender;
+- (void) checkAll;
+- (void) uncheckAll;
 - (void) setPriority: (id) sender;
 
 - (void) revealFile: (id) sender;
 
+- (void) renameSelected: (id) sender;
+
 @end
