@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: FileOutlineController.m 12483 2011-05-31 22:26:04Z livings124 $
+ * $Id$
  *
- * Copyright (c) 2008-2011 Transmission authors and contributors
+ * Copyright (c) 2008-2012 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -24,10 +24,13 @@
 
 #import "FileOutlineController.h"
 #import "Torrent.h"
+#import "FileListNode.h"
 #import "FileOutlineView.h"
 #import "FilePriorityCell.h"
-#import "FileListNode.h"
+#import "FileRenameSheetController.h"
 #import "NSApplicationAdditions.h"
+#import "NSMutableArrayAdditions.h"
+#import "NSStringAdditions.h"
 #import <Quartz/Quartz.h>
 
 #define ROW_SMALL_HEIGHT 18.0
@@ -49,20 +52,22 @@ typedef enum
 
 - (NSMenu *) menu;
 
+- (NSUInteger) findFileNode: (FileListNode *) node inList: (NSArray *) list atIndexes: (NSIndexSet *) range currentParent: (FileListNode *) currentParent finalParent: (FileListNode **) parent;
+
 @end
 
 @implementation FileOutlineController
 
 - (void) awakeFromNib
 {
+    fFileList = [[NSMutableArray alloc] init];
+    
     [fOutline setDoubleAction: @selector(revealFile:)];
     [fOutline setTarget: self];
     
     //set table header tool tips
-    [[fOutline tableColumnWithIdentifier: @"Check"] setHeaderToolTip: NSLocalizedString(@"Download",
-                                                                        "file table -> header tool tip")];
-    [[fOutline tableColumnWithIdentifier: @"Priority"] setHeaderToolTip: NSLocalizedString(@"Priority",
-                                                                        "file table -> header tool tip")];
+    [[fOutline tableColumnWithIdentifier: @"Check"] setHeaderToolTip: NSLocalizedString(@"Download", "file table -> header tool tip")];
+    [[fOutline tableColumnWithIdentifier: @"Priority"] setHeaderToolTip: NSLocalizedString(@"Priority", "file table -> header tool tip")];
     
     [fOutline setMenu: [self menu]];
     
@@ -73,6 +78,7 @@ typedef enum
 {
     [fFileList release];
     [fFilterText release];
+    
     [super dealloc];
 }
 
@@ -85,55 +91,130 @@ typedef enum
 {
     fTorrent = torrent;
     
-    [fFileList release];
-    fFileList = [[fTorrent fileList] retain];
+    [fFileList setArray: [fTorrent fileList]];
     
     [fFilterText release];
     fFilterText = nil;
     
-    [fOutline deselectAll: nil];
     [fOutline reloadData];
+    [fOutline deselectAll: nil]; //do this after reloading the data #4575
 }
 
 - (void) setFilterText: (NSString *) text
 {
-    if ([text isEqualToString: @""])
+    NSArray * components = [text betterComponentsSeparatedByCharactersInSet: [NSCharacterSet whitespaceAndNewlineCharacterSet]];
+    if (!components || [components count] == 0)
+    {
         text = nil;
+        components = nil;
+    }
     
     if ((!text && !fFilterText) || (text && fFilterText && [text isEqualToString: fFilterText]))
         return;
+
+    [fOutline beginUpdates];
+    
+    NSUInteger currentIndex = 0, totalCount = 0;
+    NSMutableArray * itemsToAdd = [NSMutableArray array];
+    NSMutableIndexSet * itemsToAddIndexes = [NSMutableIndexSet indexSet];
+    
+    NSMutableDictionary * removedIndexesForParents = nil; //ugly, but we can't modify the actual file nodes
+    
+    NSArray * tempList = !text ? [fTorrent fileList] : [fTorrent flatFileList];
+    for (FileListNode * item in tempList)
+    {
+        __block BOOL filter = NO;
+        if (components)
+        {
+            [components enumerateObjectsWithOptions: NSEnumerationConcurrent usingBlock: ^(id obj, NSUInteger idx, BOOL * stop) {
+                if ([[item name] rangeOfString: (NSString *)obj options: (NSCaseInsensitiveSearch | NSDiacriticInsensitiveSearch)].location == NSNotFound)
+                {
+                    filter = YES;
+                    *stop = YES;
+                }
+            }];
+        }
+        
+        if (!filter)
+        {
+            FileListNode * parent = nil;
+            NSUInteger previousIndex = ![item isFolder] ? [self findFileNode: item inList: fFileList atIndexes: [NSIndexSet indexSetWithIndexesInRange: NSMakeRange(currentIndex, [fFileList count]-currentIndex)] currentParent: nil finalParent: &parent] : NSNotFound;
+            
+            if (previousIndex == NSNotFound)
+            {
+                [itemsToAdd addObject: item];
+                [itemsToAddIndexes addIndex: totalCount];
+            }
+            else
+            {
+                BOOL move = YES;
+                if (!parent)
+                {
+                    if (previousIndex != currentIndex)
+                        [fFileList moveObjectAtIndex: previousIndex toIndex: currentIndex];
+                    else
+                        move = NO;
+                }
+                else
+                {
+                    [fFileList insertObject: item atIndex: currentIndex];
+                    
+                    //figure out the index within the semi-edited table - UGLY
+                    if (!removedIndexesForParents)
+                        removedIndexesForParents = [NSMutableDictionary dictionary];
+                    
+                    NSMutableIndexSet * removedIndexes = [removedIndexesForParents objectForKey: parent];
+                    if (!removedIndexes)
+                    {
+                        removedIndexes = [NSMutableIndexSet indexSetWithIndex: previousIndex];
+                        [removedIndexesForParents setObject: removedIndexes forKey: parent];
+                    }
+                    else
+                    {
+                        [removedIndexes addIndex: previousIndex];
+                        previousIndex -= [removedIndexes countOfIndexesInRange: NSMakeRange(0, previousIndex)];
+                    }
+                }
+                
+                if (move)
+                    [fOutline moveItemAtIndex: previousIndex inParent: parent toIndex: currentIndex inParent: nil];
+                
+                ++currentIndex;
+            }
+            
+            ++totalCount;
+        }
+    }
+    
+    //remove trailing items - those are the unused
+    if (currentIndex  < [fFileList count])
+    {
+        const NSRange removeRange = NSMakeRange(currentIndex, [fFileList count]-currentIndex);
+        [fFileList removeObjectsInRange: removeRange];
+        [fOutline removeItemsAtIndexes: [NSIndexSet indexSetWithIndexesInRange: removeRange] inParent: nil withAnimation: NSTableViewAnimationSlideDown];
+    }
+    
+    //add new items
+    [fFileList insertObjects: itemsToAdd atIndexes: itemsToAddIndexes];
+    [fOutline insertItemsAtIndexes: itemsToAddIndexes inParent: nil withAnimation: NSTableViewAnimationSlideUp];
+
+    [fOutline endUpdates];
     
     [fFilterText release];
     fFilterText = [text retain];
-    
-    [fFileList release];
-    if (!fFilterText)
-        fFileList = [[fTorrent fileList] retain];
-    else
-    {
-        NSMutableArray * list = [NSMutableArray arrayWithCapacity: [fTorrent fileCount]];
-        
-        for (FileListNode * node in [fTorrent flatFileList])
-            if ([[node name] rangeOfString: fFilterText options: (NSCaseInsensitiveSearch | NSDiacriticInsensitiveSearch)].location != NSNotFound)
-                [list addObject: node];
-        
-        fFileList = [[NSArray alloc] initWithArray: list];
-    }
-    
-    [fOutline reloadData];
 }
 
-- (void) reloadData
+- (void) refresh
 {
     [fTorrent updateFileStat];
-    [fOutline reloadData];
+    
+    [fOutline setNeedsDisplay: YES];
 }
 
 - (void) outlineViewSelectionDidChange: (NSNotification *) notification
 {
-    if ([NSApp isOnSnowLeopardOrBetter] && [QLPreviewPanelSL sharedPreviewPanelExists]
-        && [[QLPreviewPanelSL sharedPreviewPanel] isVisible])
-        [[QLPreviewPanelSL sharedPreviewPanel] reloadData];
+    if ([QLPreviewPanel sharedPreviewPanelExists] && [[QLPreviewPanel sharedPreviewPanel] isVisible])
+        [[QLPreviewPanel sharedPreviewPanel] reloadData];
 }
 
 - (NSInteger) outlineView: (NSOutlineView *) outlineView numberOfChildrenOfItem: (id) item
@@ -165,8 +246,7 @@ typedef enum
         return item;
 }
 
-- (void) outlineView: (NSOutlineView *) outlineView willDisplayCell: (id) cell
-            forTableColumn: (NSTableColumn *) tableColumn item: (id) item
+- (void) outlineView: (NSOutlineView *) outlineView willDisplayCell: (id) cell forTableColumn: (NSTableColumn *) tableColumn item: (id) item
 {
     NSString * identifier = [tableColumn identifier];
     if ([identifier isEqualToString: @"Check"])
@@ -181,20 +261,19 @@ typedef enum
     else;
 }
 
-- (void) outlineView: (NSOutlineView *) outlineView setObjectValue: (id) object
-        forTableColumn: (NSTableColumn *) tableColumn byItem: (id) item
+- (void) outlineView: (NSOutlineView *) outlineView setObjectValue: (id) object forTableColumn: (NSTableColumn *) tableColumn byItem: (id) item
 {
     NSString * identifier = [tableColumn identifier];
     if ([identifier isEqualToString: @"Check"])
     {
         NSIndexSet * indexSet;
-        if (([NSApp isOnSnowLeopardOrBetter] ? [NSEvent modifierFlags] : [[NSApp currentEvent] modifierFlags]) & NSAlternateKeyMask)
+        if ([NSEvent modifierFlags] & NSAlternateKeyMask)
             indexSet = [NSIndexSet indexSetWithIndexesInRange: NSMakeRange(0, [fTorrent fileCount])];
         else
             indexSet = [(FileListNode *)item indexes];
         
         [fTorrent setFileCheckState: [object intValue] != NSOffState ? NSOnState : NSOffState forIndexes: indexSet];
-        [fOutline reloadData];
+        [fOutline setNeedsDisplay: YES];
         
         [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateUI" object: nil];
     }
@@ -213,7 +292,7 @@ typedef enum
     {
         NSString * path = [fTorrent fileLocation: item];
         if (!path)
-            path = [[item path] stringByAppendingPathComponent: [item name]];
+            path = [[(FileListNode *)item path] stringByAppendingPathComponent: [(FileListNode *)item name]];
         return path;
     }
     else if ([ident isEqualToString: @"Check"])
@@ -273,7 +352,7 @@ typedef enum
         [itemIndexes addIndexes: [[fOutline itemAtRow: i] indexes]];
     
     [fTorrent setFileCheckState: state forIndexes: itemIndexes];
-    [fOutline reloadData];
+    [fOutline setNeedsDisplay: YES];
 }
 
 - (void) setOnlySelectedCheck: (id) sender
@@ -289,7 +368,21 @@ typedef enum
     [remainingItemIndexes removeIndexes: itemIndexes];
     [fTorrent setFileCheckState: NSOffState forIndexes: remainingItemIndexes];
     
-    [fOutline reloadData];
+    [fOutline setNeedsDisplay: YES];
+}
+
+- (void) checkAll
+{
+    NSIndexSet * indexSet = [NSIndexSet indexSetWithIndexesInRange: NSMakeRange(0, [fTorrent fileCount])];
+    [fTorrent setFileCheckState: NSOnState forIndexes: indexSet];
+    [fOutline setNeedsDisplay: YES];
+}
+
+- (void) uncheckAll
+{
+    NSIndexSet * indexSet = [NSIndexSet indexSetWithIndexesInRange: NSMakeRange(0, [fTorrent fileCount])];
+    [fTorrent setFileCheckState: NSOffState forIndexes: indexSet];
+    [fOutline setNeedsDisplay: YES];
 }
 
 - (void) setPriority: (id) sender
@@ -313,33 +406,48 @@ typedef enum
         [itemIndexes addIndexes: [[fOutline itemAtRow: i] indexes]];
     
     [fTorrent setFilePriority: priority forIndexes: itemIndexes];
-    [fOutline reloadData];
+    [fOutline setNeedsDisplay: YES];
 }
 
 - (void) revealFile: (id) sender
 {
     NSIndexSet * indexes = [fOutline selectedRowIndexes];
-    if ([NSApp isOnSnowLeopardOrBetter])
+    NSMutableArray * paths = [NSMutableArray arrayWithCapacity: [indexes count]];
+    for (NSUInteger i = [indexes firstIndex]; i != NSNotFound; i = [indexes indexGreaterThanIndex: i])
     {
-        NSMutableArray * paths = [NSMutableArray arrayWithCapacity: [indexes count]];
-        for (NSUInteger i = [indexes firstIndex]; i != NSNotFound; i = [indexes indexGreaterThanIndex: i])
-        {
-            NSString * path = [fTorrent fileLocation: [fOutline itemAtRow: i]];
-            if (path)
-                [paths addObject: [NSURL fileURLWithPath: path]];
-        }
-        
-        if ([paths count])
-            [[NSWorkspace sharedWorkspace] activateFileViewerSelectingURLs: paths];
+        NSString * path = [fTorrent fileLocation: [fOutline itemAtRow: i]];
+        if (path)
+            [paths addObject: [NSURL fileURLWithPath: path]];
+    }
+    
+    if ([paths count] > 0)
+        [[NSWorkspace sharedWorkspace] activateFileViewerSelectingURLs: paths];
+}
+
+- (void) renameSelected: (id) sender
+{
+    NSIndexSet * indexes = [fOutline selectedRowIndexes];
+    NSAssert([indexes count] == 1, @"1 file needs to be selected to rename, but %ld are selected", [indexes count]);
+    
+    FileListNode * node = [fOutline itemAtRow: [indexes firstIndex]];
+    Torrent * torrent = [node torrent];
+    if (![torrent isFolder])
+    {
+        [FileRenameSheetController presentSheetForTorrent: torrent modalForWindow: [fOutline window] completionHandler: ^(BOOL didRename) {
+            if (didRename)
+            {
+                [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateQueue" object: self];
+                [[NSNotificationCenter defaultCenter] postNotificationName: @"ResetInspector" object: self userInfo: @{ @"Torrent" : torrent }];
+            }
+        }];
     }
     else
     {
-        for (NSUInteger i = [indexes firstIndex]; i != NSNotFound; i = [indexes indexGreaterThanIndex: i])
-        {
-            NSString * path = [fTorrent fileLocation: [fOutline itemAtRow: i]];
-            if (path)
-                [[NSWorkspace sharedWorkspace] selectFile: path inFileViewerRootedAtPath: nil];
-        }
+        [FileRenameSheetController presentSheetForFileListNode: node modalForWindow: [fOutline window] completionHandler: ^(BOOL didRename) {
+            #warning instead of calling reset inspector, just resort?
+            if (didRename)
+                [[NSNotificationCenter defaultCenter] postNotificationName: @"ResetInspector" object: self userInfo: @{ @"Torrent" : torrent }];
+        }];
     }
 }
 
@@ -430,6 +538,11 @@ typedef enum
         return canChange;
     }
     
+    if (action == @selector(renameSelected:))
+    {
+        return [fOutline numberOfSelectedRows] == 1;
+    }
+    
     return YES;
 }
 
@@ -443,7 +556,7 @@ typedef enum
     
     //check and uncheck
     NSMenuItem * item = [[NSMenuItem alloc] initWithTitle: NSLocalizedString(@"Check Selected", "File Outline -> Menu")
-                            action: @selector(setCheck:) keyEquivalent: @""];
+            action: @selector(setCheck:) keyEquivalent: @""];
     [item setTarget: self];
     [item setTag: FILE_CHECK_TAG];
     [menu addItem: item];
@@ -476,7 +589,7 @@ typedef enum
             action: @selector(setPriority:) keyEquivalent: @""];
     [item setTarget: self];
     [item setTag: FILE_PRIORITY_HIGH_TAG];
-    [item setImage: [NSImage imageNamed: @"PriorityHigh.png"]];
+    [item setImage: [NSImage imageNamed: @"PriorityHighTemplate"]];
     [priorityMenu addItem: item];
     [item release];
     
@@ -484,7 +597,7 @@ typedef enum
             action: @selector(setPriority:) keyEquivalent: @""];
     [item setTarget: self];
     [item setTag: FILE_PRIORITY_NORMAL_TAG];
-    [item setImage: [NSImage imageNamed: @"PriorityNormal.png"]];
+    [item setImage: [NSImage imageNamed: @"PriorityNormalTemplate"]];
     [priorityMenu addItem: item];
     [item release];
     
@@ -492,7 +605,7 @@ typedef enum
             action: @selector(setPriority:) keyEquivalent: @""];
     [item setTarget: self];
     [item setTag: FILE_PRIORITY_LOW_TAG];
-    [item setImage: [NSImage imageNamed: @"PriorityLow.png"]];
+    [item setImage: [NSImage imageNamed: @"PriorityLowTemplate"]];
     [priorityMenu addItem: item];
     [item release];
     
@@ -502,7 +615,16 @@ typedef enum
     
     //reveal in finder
     item = [[NSMenuItem alloc] initWithTitle: NSLocalizedString(@"Show in Finder", "File Outline -> Menu")
-            action: @selector(revealFile:) keyEquivalent: @""];
+                                      action: @selector(revealFile:) keyEquivalent: @""];
+    [item setTarget: self];
+    [menu addItem: item];
+    [item release];
+    
+    [menu addItem: [NSMenuItem separatorItem]];
+    
+    //rename
+    item = [[NSMenuItem alloc] initWithTitle: [NSLocalizedString(@"Rename File", "File Outline -> Menu") stringByAppendingEllipsis]
+                                                   action: @selector(renameSelected:) keyEquivalent: @""];
     [item setTarget: self];
     [menu addItem: item];
     [item release];
@@ -510,4 +632,34 @@ typedef enum
     return [menu autorelease];
 }
 
+- (NSUInteger) findFileNode: (FileListNode *) node inList: (NSArray *) list atIndexes: (NSIndexSet *) indexes currentParent: (FileListNode *) currentParent finalParent: (FileListNode **) parent
+{
+    NSAssert(![node isFolder], @"Looking up folder node!");
+    
+    __block NSUInteger retIndex = NSNotFound;
+    
+    [list enumerateObjectsAtIndexes: indexes options: NSEnumerationConcurrent usingBlock: ^(id checkNode, NSUInteger index, BOOL * stop) {
+        if ([[checkNode indexes] containsIndex: [[node indexes] firstIndex]])
+        {
+            if (![checkNode isFolder])
+            {
+                NSAssert2([checkNode isEqualTo: node], @"Expected file nodes to be equal: %@ %@", checkNode, node);
+                
+                *parent = currentParent;
+                retIndex = index;
+            }
+            else
+            {
+                const NSUInteger subIndex = [self findFileNode: node inList: [checkNode children] atIndexes: [NSIndexSet indexSetWithIndexesInRange: NSMakeRange(0, [[checkNode children] count])] currentParent: checkNode finalParent: parent];
+                NSAssert(subIndex != NSNotFound, @"We didn't find an expected file node.");
+                retIndex = subIndex;
+            }
+            
+            *stop = YES;
+        }
+    }];
+    
+    return retIndex;
+}
+
 @end
