@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: PrefsController.h 11968 2011-02-18 03:50:19Z livings124 $
+ * $Id$
  *
- * Copyright (c) 2005-2011 Transmission authors and contributors
+ * Copyright (c) 2005-2012 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -27,8 +27,9 @@
 
 @class PortChecker;
 
-@interface PrefsController : NSWindowController
+@interface PrefsController : NSWindowController <NSToolbarDelegate>
 {
+    tr_session * fHandle;
     NSUserDefaults * fDefaults;
     BOOL fHasLoaded;
     
@@ -36,10 +37,12 @@
     
     NSString * fInitialString;
     
+    IBOutlet NSButton * fBuiltInGrowlButton, *fGrowlAppButton;
     IBOutlet NSTextField * fCheckForUpdatesLabel;
     IBOutlet NSButton * fCheckForUpdatesButton, * fCheckForUpdatesBetaButton;
     
     IBOutlet NSPopUpButton * fFolderPopUp, * fIncompleteFolderPopUp, * fImportFolderPopUp, * fDoneScriptPopUp;
+    IBOutlet NSButton * fShowMagnetAddWindowCheck;
     IBOutlet NSTextField * fRatioStopField, * fIdleStopField, * fQueueDownloadField, * fQueueSeedField, * fStalledField;
 
     IBOutlet NSTextField * fUploadField, * fDownloadField,
@@ -65,8 +68,7 @@
     NSString * fRPCPassword;
 }
 
-+ (void) setHandle: (tr_session *) handle;
-+ (tr_session *) handle;
+- (id) initWithHandle: (tr_session *) handle;
 
 - (void) setAutoUpdateToBeta: (id) sender;
 
@@ -102,6 +104,10 @@
 
 - (void) setBadge: (id) sender;
 
+- (IBAction) setBuiltInGrowlEnabled: (id) sender;
+- (IBAction) openGrowlApp: (id) sender;
+- (void) openNotificationSystemPrefs: (id) sender;
+
 - (void) resetWarnings: (id) sender;
 
 - (void) setDefaultForMagnets: (id) sender;
@@ -119,15 +125,20 @@
 
 - (void) setRenamePartialFiles: (id) sender;
 
+- (IBAction) setShowAddMagnetWindow: (id) sender;
+- (void) updateShowAddMagnetWindowField;
+
 - (void) setDoneScriptEnabled: (id) sender;
 - (void) doneScriptSheetShow: (id) sender;
 
 - (void) applyRatioSetting: (id) sender;
 - (void) setRatioStop: (id) sender;
 - (void) updateRatioStopField;
+- (void) updateRatioStopFieldOld;
 
 - (void) applyIdleStopSetting: (id) sender;
 - (void) setIdleStop: (id) sender;
+- (void) updateLimitStopField;
 
 - (void) applySpeedSettings: (id) sender;
 - (void) applyAltSpeedSettings;
