@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: TorrentGroup.m 10522 2010-04-23 16:59:14Z charles $
+ * $Id$
  * 
- * Copyright (c) 2008-2010 Transmission authors and contributors
+ * Copyright (c) 2008-2012 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -23,6 +23,7 @@
  *****************************************************************************/
 
 #import "TorrentGroup.h"
+#import "GroupsController.h"
 #import "Torrent.h"
 
 #include "transmission.h" // required by utils.h
@@ -46,11 +47,21 @@
     [super dealloc];
 }
 
+- (NSString *) description
+{
+    return [NSString stringWithFormat: @"Torrent Group %ld: %@", fGroup, fTorrents];
+}
+
 - (NSInteger) groupIndex
 {
     return fGroup;
 }
 
+- (NSInteger) groupOrderValue
+{
+    return [[GroupsController groups] rowValueForIndex: fGroup];
+}
+
 - (NSMutableArray *) torrents
 {
     return fTorrents;
