@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: TrackerTableView.h 7659 2009-01-10 23:37:37Z livings124 $
+ * $Id$
  * 
- * Copyright (c) 2008-2009 Transmission authors and contributors
+ * Copyright (c) 2008-2012 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -24,11 +24,19 @@
 
 #import <Cocoa/Cocoa.h>
 
+@class Torrent;
+
 @interface TrackerTableView : NSTableView
 {
+    //weak references
+    Torrent * fTorrent;
     NSArray * fTrackers;
 }
 
+- (void) setTorrent: (Torrent *) torrent;
 - (void) setTrackers: (NSArray *) trackers;
 
+- (void) copy: (id) sender;
+- (void) paste: (id) sender;
+
 @end
