@@ -20,39 +20,41 @@ peer_id_prefix=`grep m4_define configure.ac | sed "s/[][)(]/,/g" | grep peer_id_
 major_version=`echo ${user_agent_prefix} | awk -F . '{print $1}'`
 minor_version=`echo ${user_agent_prefix} | awk -F . '{print $2 + 0}'`
 
-svn_revision_file=REVISION
-svn_revision_reliable=true
+vcs_revision=
+vcs_revision_file=REVISION
 
-if [ -n "$JENKINS_URL" -a -n "$SVN_REVISION" ]; then
-    # Jenkins automated build, use the set environment variables to avoid
-    # version mismatches between java's svn and command line's svn
-    svn_revision=$SVN_REVISION
-elif [ -d ".svn" ] && type svnversion >/dev/null 2>&1; then
-    # If this is a svn tree, and svnversion is available in PATH, use it to
-    # grab the version.
-    svn_revision=`svnversion -n . | cut -d: -f1 | cut -dM -f1 | cut -dS -f1`
-elif [ -f "$svn_revision_file" ]; then
-    svn_revision=`cat "$svn_revision_file"`
-else
-    # Give up and check the source files
-    svn_revision=`awk '/\\$Id: /{ if ($4>i) i=$4 } END {print i}' */*.cc */*.[chm] */*.po`
-    svn_revision_reliable=false
+if [ -n "$JENKINS_URL" -a -n "$GIT_COMMIT" ]; then
+    vcs_revision=$GIT_COMMIT
+elif [ -n "$TEAMCITY_PROJECT_NAME" -a -n "$BUILD_VCS_NUMBER" ]; then
+    vcs_revision=$BUILD_VCS_NUMBER
+elif [ -d ".git" ] && type git >/dev/null 2>&1; then
+    vcs_revision=`git rev-list --max-count=1 HEAD`
+elif [ -f "$vcs_revision_file" ]; then
+    vcs_revision=`cat "$vcs_revision_file"`
 fi
 
-if $svn_revision_reliable; then
-    [ -f "$svn_revision_file" ] && [ "`cat "$svn_revision_file"`" -eq "$svn_revision" ] || echo "$svn_revision" > "$svn_revision_file"
+vcs_revision=`echo $vcs_revision`
+
+if [ -n "$vcs_revision" ]; then
+    [ -f "$vcs_revision_file" ] && [ "`cat "$vcs_revision_file"`" = "$vcs_revision" ] || echo "$vcs_revision" > "$vcs_revision_file"
 else
-    rm -f "$svn_revision_file"
+    vcs_revision=0
+    rm -f "$vcs_revision_file"
 fi
 
+vcs_revision=`echo ${vcs_revision} | head -c10`
+
 cat > libtransmission/version.h.new << EOF
+#pragma once
+
 #define PEERID_PREFIX             "${peer_id_prefix}"
 #define USERAGENT_PREFIX          "${user_agent_prefix}"
-#define SVN_REVISION              "${svn_revision}"
-#define SVN_REVISION_NUM          ${svn_revision}
+#define VCS_REVISION              "${vcs_revision}"
+#define VCS_REVISION_NUM          ${vcs_revision}
 #define SHORT_VERSION_STRING      "${user_agent_prefix}"
-#define LONG_VERSION_STRING       "${user_agent_prefix} (${svn_revision})"
+#define LONG_VERSION_STRING       "${user_agent_prefix} (${vcs_revision})"
 #define VERSION_STRING_INFOPLIST  ${user_agent_prefix}
+#define BUILD_STRING_INFOPLIST    14714.${major_version}.${minor_version}
 #define MAJOR_VERSION             ${major_version}
 #define MINOR_VERSION             ${minor_version}
 EOF
